
import os
import subprocess
import tempfile

import colomoto.setup_helper

class CmdError(subprocess.CalledProcessError):
    def __str__(self):
        stderr = "\n%s" % self.stderr.decode()
        return "Command '%s' returned non-zero exit status %d\n%s\n" \
            % (" ".join(self.cmd), self.returncode, stderr)

class NuSMVResults(dict):
    def alltrue(self):
        return False not in self.values()

class NuSMV(object):
    def __init__(self, input_smv):
        self.input_smv = input_smv
        self.__custom_specs = []
        self.append_lines = []
        self.opts = {
            "dcx": True,
            "coi": True,
        }

    def check_output(self, timeout=None):
        """
        Call NuSMV and returns the content of stdout
        """
        if self.append_lines:
            # we need to generate a temporary file
            fd, input_file = tempfile.mkstemp(suffix=".smv")
            with open(fd, "w") as out, open(self.input_smv) as src:
                out.write(src.read())
                for line in self.append_lines:
                    out.write("%s\n" % line)
            todel = True
        else:
            input_file = self.input_smv
            todel = False
        args = ["NuSMV"]
        for k, v in self.opts.items():
            if isinstance(v, bool) and v:
                args.append("-%s" % k)
            else:
                raise NotImplementedError
        args.append(input_file)

        try:
            output = subprocess.check_output(args, stderr=subprocess.PIPE,
                        timeout=timeout).decode().strip()
        except subprocess.CalledProcessError as e:
            raise CmdError(e.returncode, e.cmd, e.output, e.stderr) from None
        finally:
            if todel:
                os.unlink(input_file)
        return output

    def _parse_output(self, output, as_dict=True):
        results = []
        for line in output.split("\n"):
            if not line.startswith("-- specification "):
                continue
            parts = line.split()
            valid = True if parts[-1] == "true" else False
            spec = " ".join(parts[2:-2])
            results.append((spec, valid) if as_dict else valid)

        if as_dict and self.__custom_specs:
            nc = len(self.__custom_specs)
            results = results[:-nc] \
                + list(zip(self.__custom_specs, [valid for (_,valid) in results[-nc:]]))

        if as_dict:
            return NuSMVResults(results)
        else:
            return results

    def verify(self, timeout=None, as_dict=True):
        output = self.check_output()
        return self._parse_output(output, as_dict=as_dict)

    def alltrue(self, timeout=None):
        return self.verify().alltrue()

    def add_instruction(self, line):
        self.append_lines.append(line)

    def add_spec(self, tspec, expr, *fmt, **kwargs):
        self.modified = True
        assert tspec in ["LTL", "CTL"], "Unknown specification type"
        name = kwargs.get("name")
        if fmt:
            vfmt = []
            kfmt = {}
            for v in fmt:
                if isinstance(v, dict):
                    kfmt.update(v)
                else:
                    vfmt.append(v)
            expr = expr.format(*vfmt, **kfmt)
        self.add_instruction("{}SPEC {}{};\n".format(tspec,
                    "NAME {} := ".format(name) if name else "",
                    expr))
        self.__custom_specs.append(name if name else expr)

    def add_ltl(self, expr, *fmt, **kwargs):
        self.add_spec("LTL", expr, *fmt, **kwargs)

    def add_ltls(self, specs, *fmt, **kwargs):
        for name, expr in specs.items():
            kwargs["name"] = name
            self.add_ltl(expr, *fmt, **kwargs)

    def add_ctl(self, expr, *fmt, **kwargs):
        self.add_spec("CTL", expr, *fmt, **kwargs)

    def add_ctls(self, specs, *fmt, **kwargs):
        for name, expr in specs.items():
            kwargs["name"] = name
            self.add_ctl(expr, *fmt, **kwargs)

    def add_init(self, expr):
        self.add_instruction("INIT {}".format(expr))

    def reset(self):
        self.append_lines.clear()
        self.__custom_specs.clear()

def load(filename):
    return NuSMV(filename)





import re

from urllib.parse import urlparse

from xml.dom import *
from xml.dom.minidom import parse

from bs4 import BeautifulSoup

from colomoto_jupyter import *

if IN_IPYTHON:
    from IPython.display import Markdown

urlidentifier = re.compile("https?://[^/]*\\bcellcollective\.org/[^/]*#(\\d+)\\b")

def id_from_url(url):
    uri = urlparse(url)
    if uri.netloc:
        if uri.scheme == "cellcollective":
            return uri.netloc
        urlmatch = urlidentifier.search(url)
        if urlmatch:
            return urlmatch.group(1)

def url_matches(url):
    return id_from_url(url) is not None

class CellCollectiveConnector(object):
    def __init__(self, identifier):
        self.id = id_from_url(identifier) or identifier
    @property
    def sbml_url(self):
        return "http://api.cellcollective.org/model/export/{}?type=SBML".format(self.id)
    @property
    def sbml_basename(self):
        return "{}.sbml".format(self.id)

def connect(identifier):
    return CellCollectiveConnector(identifier)


METADATA_UNITPROTID = "UniProtID"
METADATA_GENENAME = "GeneName"
METADATA_NCBIGENEID = "NCBIGeneID"

QUALNS = "http://www.sbml.org/sbml/level3/version1/qual/version1"

class CellCollectiveSBMLModel(object):
    def __init__(self, localfile):
        self.localfile = localfile
        self.dom = parse(localfile)
        self.root = self.dom.documentElement
        species_elts = self.root.getElementsByTagNameNS(QUALNS, "qualitativeSpecies")
        self.id2elt = dict([(e.getAttributeNS(QUALNS, "id"), e) \
                                for e in species_elts])
        self.name2id = dict([(e.getAttributeNS(QUALNS, "name"), id) \
                                for id, e in self.id2elt.items()])

    @property
    def species(self):
        """
        Returns the set of defined species

        :rtype: set
        """
        return set(self.name2id.keys())

    _key2metadata = {
        "uniprotid": METADATA_UNITPROTID,
        "uniprotaccessionid": METADATA_UNITPROTID,
        "genename": METADATA_GENENAME,
        "ncbigeneid": METADATA_NCBIGENEID,
    }

    def species_metadata(self, name):
        metadata = {}
        notes = self.id2elt[self.name2id[name]].getElementsByTagName("notes")
        bodies = notes[0].getElementsByTagName("body") if notes else None
        ps = bodies[0].getElementsByTagName("p") if bodies else None
        if not ps:
            return metadata
        htmldata = BeautifulSoup(ps[0].firstChild.wholeText, "html.parser")

        def parse_statement(data):
            t = data.split(":")
            if len(t) == 2:
                key = t[0].strip().replace(" ","").lower()
                value = t[1].strip()
                if key in self._key2metadata:
                    metadata[self._key2metadata[key]] = value

        divs = htmldata.find_all("div")
        for div in divs:
            parse_statement(div.getText())
        if not divs:
            for p in ps:
                parse_statement(p.firstChild.wholeText)

        return metadata

    def species_uniprotkb(self, name):
        uniprotid = self.species_metadata(name).get(METADATA_UNITPROTID)
        if not uniprotid:
            return
        return URL("https://www.uniprot.org/uniprot/%s" % uniprotid)

    def species_ncbi_gene(self, name):
        id = self.species_metadata(name).get(METADATA_NCBIGENEID)
        if not id:
            return
        return URL("https://www.ncbi.nlm.nih.gov/gene/%s" % id)



def load(identifier):
    conn = None
    if isinstance(identifier, CellCollectiveConnector):
        conn = identifier
    elif url_matches(identifier):
        conn = CellCollectiveConnector(identifier)
    else:
        from colomoto_jupyter.io import ensure_localfile
        sbmlfile = ensure_localfile(identifier)
    if conn:
        from colomoto_jupyter.io import download
        url = conn.sbml_url
        bname = conn.sbml_basename
        sbmlfile = download(url, suffix=bname)
    return CellCollectiveSBMLModel(sbmlfile)

def to_biolqm(model):
    biolqm = import_colomoto_tool("biolqm")
    lqm = biolqm.load(model.localfile)
    return biolqm.sanitize(lqm)





import os
from urllib.parse import urlparse
from urllib.request import urlretrieve

from .sessionfiles import new_output_file
from .ui import info

import cellcollective

def download(url, suffix=None):
    filename = new_output_file(suffix=suffix)
    info("Downloading %s" % url)
    filename, _ = urlretrieve(url, filename=filename)
    return filename

def ensure_localfile(filename):
    uri = urlparse(filename)
    if uri.netloc:
        if cellcollective.url_matches(filename):
            cellc = cellcollective.connect(filename)
            bname = cellc.sbml_basename
            url = cellc.sbml_url
        else:
            bname = os.path.basename(filename)
            url = uri.geturl()
        filename = download(url, suffix=bname)
    assert os.path.exists(filename)
    return filename





def multivalue_merge(a,b):
    if a == b:
        return a
    if a == "*" or b == "*":
        return "*"
    mv = set()
    for x in [a,b]:
        if isinstance(x,int):
            mv.add(x)
        else:
            mv.update(x)
    return tuple(mv)

class PartialState(dict):
    def match_state(self, s):
        for k, v in self.items():
            if v != "*" and s[k] != v:
                return False
        return True
    def project(self, keys):
        return dict([(k,v) for k,v in self.items() if k in keys])

class State(PartialState):
    pass

class TrapSpaceAttractor(dict):
    def extend(self, ts):
        if not isinstance(ts, TrapSpaceAttractor):
            ts = TrapSpaceAttractor(ts)
        return TrapSpacesAttractor([self, ts])
    def match_partial_state(self, ps):
        for k, v in ps.items():
            av = self.get(k)
            if av != "*" and av != v:
                return False
        return True
    def project(self, keys):
        return dict([(k,v) for k,v in self.items() if k in keys])

    @property
    def is_single_state(self):
        for v in self.values():
            if not isinstance(v,int):
                return False
        return True

class TrapSpacesAttractor(list):
    def extend(self, ts):
        if not isinstance(ts, TrapSpaceAttractor):
            ts = TrapSpaceAttractor(ts)
        return TrapSpacesAttractor(self+[ts])
    def match_partial_state(self, ps):
        for tsa in self:
            if tsa.match_partial_state(ps):
                return True
        return False
    def project(self, keys):
        p = self[0].projec(keys)
        for tp in self[1:]:
            for k,v in tp.project(keys).items():
                p[k] = multivalue_merge(p[k], v)
        return p

    @property
    def is_single_state(self):
        return False





from .temporal_logics import nusmv_of_expr, its_of_expr

from nusmv import NuSMV
from itstools import ITSModel

def default_state_tr(ai):
    return "{}={}".format(*ai)

class ColomotoNuSMV(NuSMV):
    def __init__(self, filename, tr=default_state_tr):
        NuSMV.__init__(self, filename)
        self.tr = tr

    def add_spec(self, tspec, expr, **kwargs):
        expr = nusmv_of_expr(expr, self.tr)
        super(ColomotoNuSMV, self).add_spec(tspec, expr, **kwargs)

class ColomotoITS(ITSModel):
    def __init__(self, filename, tr=default_state_tr, **kwargs):
        ITSModel.__init__(self, filename, **kwargs)
        self.tr = tr

    def reachability(self, state_formula, **kwargs):
        state_formula = its_of_expr(state_formula, self.tr)
        return super(ColomotoITS, self).reachability(state_formula, **kwargs)

    def verify_ctls(self, specs, **kwargs):
        if isinstance(specs, dict):
            for k in specs:
                specs[k] = its_of_expr(specs[k], self.tr)
        else:
            specs = [its_of_expr(ctl, self.tr) for ctl in specs]
        return super(ColomotoITS, self).verify_ctls(specs, **kwargs)





import os
import json
import base64

from .formatters import install_default_formatters
from .wui import wui_sources

basedir = os.path.join(os.path.dirname(__file__))

try:
    get_ipython()
    IN_IPYTHON = True
except NameError:
    IN_IPYTHON = False

if IN_IPYTHON:
    from IPython.display import display, HTML, Image, Markdown

    def hello():
        docker_image = os.getenv("DOCKER_IMAGE")
        if docker_image:
            docker_date = os.getenv("DOCKER_BUILD_DATE")

            docker_name, docker_tag = docker_image.split(":")

            # remove global namespace
            parts = docker_name.split("/")
            if len(parts) > 2:
                docker_name = "/".join(parts[-2:])

            if docker_tag in ["next", "latest"] and docker_date:
                label = "`{}` built on `{}`".format(docker_name, docker_date)
            else:
                label = "`{}:{}`".format(docker_name, docker_tag)

            msg = "This notebook has been executed using the docker image %s" % label
            display(Markdown(msg))

    hello()


if IN_IPYTHON:
    def URL(url):
        return Markdown("[{0}]({0})".format(url))
else:
    def URL(url):
        return url

def import_colomoto_tool(modname):
    """
    Import the module `modname` and make it available globally when in IPython.

    Usage:

    >>> modname = import_colomoto_tool("modname")
    """
    mod = __import__(modname, globals(), locals())
    if IN_IPYTHON:
        import builtins
        attr = modname.split(".")[0]
        if not hasattr(builtins, attr):
            setattr(builtins, attr, mod)
    return mod

def jupyter_js(data, autoclean=True, **args):
    if autoclean:
        args["class"] = "to-be-removed"
    args = " ".join(['%s="%s"' % it for it in args.items()]) if args else ""
    return """<script type="text/javascript" %s>
        if (typeof Jupyter != 'undefined') {
            %s }</script>""" % (args, data)

def disp_jupyter_js(data, **opts):
    display(HTML(jupyter_js(data, **opts)))

__GLOBAL_INSTALL_DONE = False
def jupyter_setup(*args, **kwargs):
    js_src = ""
    css_src = ""
    global __GLOBAL_INSTALL_DONE
    if not __GLOBAL_INSTALL_DONE:
        jsfile = os.path.join(basedir, "jupyter_ext.js")
        install_default_formatters()
        with open(jsfile) as f:
            js_src = f.read()
        __GLOBAL_INSTALL_DONE = True

    jsargs = {}
    wui_src = wui_sources(*args, **kwargs)
    js_src += wui_src["js"]
    css_src += wui_src["css"]
    if "ssid" in wui_src:
        jsargs["id"] = wui_src["ssid"]

    js_src = jupyter_js(js_src, **jsargs)
    display(HTML("%s%s" % (js_src, css_src)))

def show_image(data):
    if isinstance(data,str):
        data = base64.b64decode(data)
    return Image(data=data)

if IN_IPYTHON:
    try:
        from tabulate import tabulate as _tabulate
        def tabulate(*args, **kwargs):
            defaults = {
                "tablefmt": "html",
                "headers": "keys",
                "showindex": "always"
            }
            defaults.update(kwargs)
            return Markdown(_tabulate(*args, **defaults))
    except ImportError:
        pass





try:
    import networkx as nx
    HAS_NETWORKX = True
except ImportError:
    HAS_NETWORKX = False

def svg_of_graph(g):
    """
    Returns SVG representation of ``networkx.Graph`` `g` with GraphViz dot layout.
    """
    print("# computing graph layout...")
    return nx.nx_pydot.to_pydot(g).create_svg().decode()


def install_default_formatters():
    """
    Register default IPython formatters:

    * ``networkx.Graph`` with :py:func:`.svg_of_graph`
    """
    ip = get_ipython()
    if HAS_NETWORKX:
        # nxgraph to svg
        svg_formatter = ip.display_formatter.formatters["image/svg+xml"]
        svg_formatter.for_type(nx.Graph, svg_of_graph)





import copy

from colomoto_jupyter import import_colomoto_tool
from colomoto_jupyter.sessionfiles import new_output_file
from colomoto_jupyter import IN_IPYTHON, jupyter_setup

import boolean

if IN_IPYTHON:
    jupyter_setup("minibn", label="miniBN")

class NOT(boolean.NOT):
    def __init__(self, *args):
        super().__init__(*args)
        self.operator = "!"


class BaseNetwork(dict):
    def __init__(self, data=None, Symbol_class=boolean.Symbol,
            allowed_in_name=('.','_',':','-'), **kwargs):
        super().__init__()
        self.ba = boolean.BooleanAlgebra(NOT_class=NOT,
            Symbol_class=Symbol_class,
            allowed_in_token=allowed_in_name)
        if data:
            if isinstance(data, str):
                self.import_data(data.split("\n"))
            elif isinstance(data, dict):
                for a, f in data.items():
                    self[a] = f
            else:
                self.import_data(data)
        for a, f in kwargs.items():
            self[a] = f

    def import_data(self, data):
        raise NotImplementedError
    def source(self, sep):
        raise NotImplementedError
    def __repr__(self):
        return self.source(sep=" <- ")

    @classmethod
    def load(celf, filename):
        f = celf()
        with open(filename) as data:
            f.import_data(data)
        return f


    def v(self, name):
        return self.ba.symbols(name)[0]
    def vars(self, *names):
        return self.ba.symbols(*names)

    def _autokey(self, a):
        if isinstance(a, self.ba.Symbol):
            a = a.obj
        return a

    def _autobool(self, expr):
        if isinstance(expr, bool):
            return self.ba.TRUE if expr else self.ba.FALSE
        elif isinstance(expr, int):
            return self.ba.TRUE if expr > 0 else self.ba.FALSE
        return expr

    def _normalize_tr(self, tr):
        ntr = {}
        for k, v in tr.items():
            if not isinstance(k, self.ba.Symbol):
                k = self.v(k)
            ntr[k] = self._autobool(v)
        return ntr

    def rewrite(self, a, tr):
        tr = self._normalize_tr(tr)
        self[a] = self[a].subs(tr).simplify()

    def __setitem__(self, a, f):
        if isinstance(f, str):
            f = self.ba.parse(f)
        f = self._autobool(f)
        return super().__setitem__(self._autokey(a), f)

    def __getitem__(self, a):
        return super().__getitem__(self._autokey(a))

    def copy(self):
        bn = copy.copy(self)
        bn.ba = self.ba
        return bn

    biolqm_format = None
    def to_biolqm(self):
        bnfile = new_output_file(ext=self.biolqm_format)
        with open(bnfile, "w") as f:
            f.write(self.source())
        biolqm = import_colomoto_tool("biolqm")
        return biolqm.load(bnfile)

class BooleanNetwork(BaseNetwork):

    biolqm_format = "bnet"
    def source(self, sep=", "):
        buf = ""
        for a, f in sorted(self.items()):
            if f is self.ba.TRUE:
                f = 1
            elif f is self.ba.FALSE:
                f = 0
            buf += "{}{}{}\n".format(a, sep, f)
        return buf

    def inputs(self):
        return [a for a, f in self.items() if f == self.v(a)]

    def import_data(self, data):
        header = None
        for line in data:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            sep = line.find("<-")
            if sep < 0:
                comma = line.index(",")
                left = line[:comma].strip()
                right = line[comma+1:].strip()
            else:
                left = line[:sep].strip()
                right = line[sep+2:].strip()
            if header is None and (left, right) == ("targets", "factors"):
                header = True
                continue
            self[left] = self.ba.parse(right)

    def simplify(self):
        bn = copy.copy(self)
        for a, f in bn.items():
            bn[a] = f.simplify()
        return bn

    def as_dnf(self):
        def make_lit(l):
            if isinstance(l, self.ba.NOT):
                return (l.args[0].obj, False)
            else:
                return (l.obj, True)
        def make_clause(c):
            if isinstance(c, self.ba.AND):
                lits = c.args
            else:
                lits = [c]
            return list(map(make_lit, lits))
        def make_dnf(f):
            if f is self.ba.TRUE:
                return True
            elif f is self.ba.FALSE:
                return False
            clauses = self.ba.dnf(f)
            if not isinstance(clauses, self.ba.OR):
                clauses = [clauses]
            else:
                clauses = clauses.args

            return list(map(make_clause, clauses))
        return dict([(i,make_dnf(f)) for (i,f) in self.items()])

    def influence_graph(self):
        import networkx as nx
        influences = set()
        ig = nx.MultiDiGraph()
        for a, f in self.items():
            ig.add_node(a)
            for lit in f.simplify().literalize().get_literals():
                if isinstance(lit, boolean.NOT):
                    b = lit.args[0].obj
                    sign = -1
                else:
                    b = lit.obj
                    sign = 1
                influences.add((b,a,sign))
        for (b, a, sign) in influences:
            ig.add_edge(b, a, sign=sign, label="+" if sign > 0 else "-")
        return ig

    def propagate_constants(self):
        csttypes = [boolean.boolean._TRUE, boolean.boolean._FALSE]
        bn = self.copy()
        csts = dict([(i,f) for i, f in bn.items() if type(f) in csttypes])
        while csts:
            new_csts = {}
            for a in bn.keys():
                if a in csts:
                    continue
                bn.rewrite(a, csts)
                if type(bn[a]) in csttypes:
                    new_csts[a] = bn[a]
            for a in csts:
                del bn[a]
            csts = new_csts
        return bn

    def to_pint(self):
        pypint = import_colomoto_tool("pypint")
        from pypint.converters import import_minibn
        return import_minibn(self)


class MVVar(boolean.Symbol):
    def __init__(self, obj):
        if isinstance(obj, str):
            parts = obj.split(":")
            if len(parts) > 1:
                try:
                    level = int(parts[-1])
                    obj = (":".join(parts[:-1]), level)
                except ValueError:
                    pass
        super().__init__(obj)
    def is_instanciated(self):
        return isinstance(self.obj, tuple)
    def nodevar(self):
        if self.is_instanciated():
            return self.obj[0]
        return self
    def level(self):
        assert self.is_instanciated()
        return self.obj[1]
    def __str__(self):
        if self.is_instanciated():
            return "{}:{}".format(*self.obj)
        else:
            return "{}".format(self.obj)
    def __truediv__(self, i):
        assert not self.is_instanciated() and isinstance(i, int), repr(self.obj)
        return self.Symbol((self.obj, i))
    def __lt__(a, b):
        assert isinstance(a, MVVar)
        if isinstance(b, MVVar):
            def getobj(mv):
                return mv.obj if mv.is_instanciated() else (mv.obj, 1)
            return getobj(a) < getobj(b)
        if isinstance(b, int):
            return ~(a / b)
        raise NotImplemented
    def __le__(a, b):
        assert isinstance(a, MVVar) and isinstance(b, int)
        return ~(a/(b+1))
    def __gt__(a, b):
        assert isinstance(a, MVVar) and isinstance(b, int)
        return a/(b+1)
    def __ge__(a, b):
        assert isinstance(a, MVVar) and isinstance(b, int)
        return a/b
    def __eq__(a, b):
        if isinstance(a, MVVar) and isinstance(b, int):
            return (a/b) & ~(a/(b+1))
        return super().__eq__(b)
    def __ne__(a, b):
        if isinstance(a, MVVar) and isinstance(b, int):
            return ~(a/b) | (a/(b+1))
        return super().__eq__(b)
    def __hash__(self):
        return super().__hash__()

class MultiValuedNetwork(BaseNetwork):
    biolqm_format = "mnet"
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, Symbol_class=MVVar)

    def _normalize(self, a, spec):
        (va,) = self.vars(a)
        if isinstance(spec, dict):
            return [(va/i, f)  for i, f in sorted(spec.items())]
        elif isinstance(spec, boolean.Expression):
            return [(va, spec)]
        return spec

    def rewrite(self, a, tr):
        tr = self._normalize_tr(tr)
        k = self._autokey(a.nodevar())
        spec = self[k]
        def _rewrite(expr):
            return expr.subs(tr).simplify()
        if isinstance(spec, boolean.Expression):
            super().rewrite(k, tr)
        elif isinstance(spec, dict):
            if a.is_instanciated():
                spec[a.level()] = _rewrite(spec[a.level()])
            else:
                for k, f in spec.items():
                    spec[k] = _rewrite(f)
        else:
            if a.is_instanciated():
                def d_rewrite(df):
                    (d,f) = df
                    if d == a:
                        f = _rewrite(f)
                    return (d,f)
            else:
                def d_rewrite(df):
                    (d,f) = df
                    return (d, _rewrite(f))
            self[k] = list(map(d_rewrite, self[k]))

    def append(self, a, f):
        if isinstance(a, str):
            (a,) = self.vars(a)
        if isinstance(f, str):
            f = self.ba.parse(f)
        k = self._autokey(a.nodevar())
        self[k] = self._normalize(k, self[k]) if k in self else []
        self[k].append((a, f))

    def source(self, sep=" <- "):
        buf = ""
        for a, f in sorted(self.items()):
            for d, g in self._normalize(a, f):
                buf += "{}{}{}\n".format(d, sep, g)
        return buf

    def import_data(self, data):
        for line in data:
            line = line.strip()
            if not line:
                continue
            left, right = line.split("<-")
            self.append(left.strip(), right.strip())






from colomoto_jupyter import IN_IPYTHON, jupyter_setup

if IN_IPYTHON:
    from IPython.display import display, FileLink

    menu = [
        {"name": "Load model",
            "snippet": "model = demo_tool.load(\"filename\")"},
        {"name":"Upload model",
            "snippet": "model = demo_tool.upload()"}, # use lists for multiple lines
        "---",
        {"name":"Model export",
            "sub-menu": [
            {"name": "Format 1 (.f1)",
                "snippet": 'model.export("model.f1")'},
            {"name": "Format 2 (.f2)",
                "snippet": 'model.export("model.f2")'},
            ]},
    ]

    toolbar = [
        {"name": "upload",
         "setup": {
            "icon": "fa-upload",
            "help": "Upload model",
            "handler": "action_upload_model"}},
    ]
    js_api = {
        "action_upload_model":
            """function () {
            var cell = Jupyter.notebook.get_selected_cell();
            cell.set_text('model = '+demo_tool_jsapi.module_alias+'.upload()');
            cell.focus_editor(); }
            """
    }
    jupyter_setup(__name__,
        label="Test",
        color="blue",
        menu=menu,
        toolbar=toolbar,
        js_api=js_api
    )

class DummyModel:
    def __init__(self, filename):
        self.filename = filename
    def __repr__(self):
        return "<DummyModel: %s>" % self.filename
    def export(self, output):
        with open(output, "w") as out:
            out.write("TEST")
        if IN_IPYTHON:
            display(FileLink(output))

def load(filename):
    return DummyModel(filename)

if IN_IPYTHON:
    from colomoto_jupyter.upload import jupyter_upload
    def upload():
        return jupyter_upload(upload, load)






import json

from . import IN_IPYTHON

if IN_IPYTHON:
    from IPython.display import display, Javascript
    from IPython.utils.py3compat import str_to_bytes, bytes_to_str

def jupyter_replace_cell_call(orig_func, dest_func, append_args=(),
                                comment=True):
        args = list(map(json.dumps, append_args))
        display(Javascript("""var __cell = Jupyter.notebook.get_selected_cell();
        colomoto_replace_call(__cell, "{}", "{}", {}, {});
        Jupyter.notebook.execute_cell_and_select_below();
        """.format(orig_func, dest_func, json.dumps(args), json.dumps(comment))))






import sys

from colomoto_jupyter import IN_IPYTHON

if IN_IPYTHON:
    from IPython.display import display, Markdown


if IN_IPYTHON:
    def info(msg):
        """
        Display `msg` (interpreted in Markdown format)
        """
        display(Markdown(msg))
else:
    def info(msg):
        """
        Print `msg` in stderr
        """
        print(msg, file=sys.stderr)





import os
import tempfile

from colomoto_jupyter import IN_IPYTHON

CFG = {
    "output_dir": tempfile.gettempdir(),
    "use_relpath": False,
    "autoclean": True,
}
"""
Python module configuation:

* `output_dir`: directory to use for saving intermediary files.
* `autoclean`: if True, intermediary files will be deleted at script exit
"""

__TMPFILES = []

def preserve_output_files(output_dir="gen"):
    """
    Make intermediate output files in directory `output_dir` instead of default
    system temporary directory, and do not remove them when the script
    terminates.
    """
    CFG["output_dir"] = dest
    CFG["autoclean"] = False

def output_dir():
    """
    Creates the output directory and returns its path
    """
    if not os.path.exists(CFG["output_dir"]):
        os.makedirs(CFG["output_dir"])
    return CFG["output_dir"]


def new_output_file(ext=None, **tempargs):
    """
    Creates a new file in :py:func:`output_dir` using `tempfile.mkstemp
    <https://docs.python.org/3/library/tempfile.html#tempfile.mkstemp>`_ and
    returns its path.
    The parameter `ext` specifies the extension of the file;
    `tempargs` are forwarded to ``tempfile.mkstemp`` with ``prefix=colomoto`` by
    default.
    """
    if "prefix" not in tempargs:
        tempargs["prefix"] = "colomoto"
    if ext is not None:
        tempargs["suffix"] = "%s.%s" % (tempargs.get("suffix", ""), ext)
    fd, filename = tempfile.mkstemp(dir=output_dir(), **tempargs)
    os.close(fd)
    __TMPFILES.append(filename)
    if CFG["use_relpath"]:
        filename = os.path.relpath(filename)
    else:
        filename = os.path.abspath(filename)
    return filename

def remove_output_files():
    """
    Removes files created by colomoto if `autoclean=True`, and in particular if
    :py:func:`preserve_output_files` has not been called.
    """
    if not CFG["autoclean"]:
        return
    for filename in __TMPFILES:
        if os.path.exists(filename):
            os.unlink(filename)
    __TMPFILES.clear()

import atexit
atexit.register(remove_output_files)

__all__ = [
    "output_dir",
    "new_output_file",
    "remove_output_files",
    "preserve_output_files",
]

